#!/bin/perl
############################################
# NIC Monitor
################
# Copyright 2011
################
# Usage:
#
# perl nicmon.pl
#
# Purpose:
#
# The program monitors all known interfaces
# Data appears under node "NIC"
# Based on netstat.pl by Sylvain Gibassier
# updated to linux and our netstat by Juraj July 2012
###########################################
use FindBin;
use lib ("$FindBin::Bin", "$FindBin::Bin/lib/perl", "$FindBin::Bin/../lib/perl");
use Wily::PrintMetric;

use strict;

my $host=`uname -n`;
my $nicmon_log='/cust/wily-tcat/logs/nicmon.log';

# define iostat command to be executed
my $netstatCommand='netstat -i';
# hold results
my @nicResults=`$netstatCommand`;

#Iface       MTU Met    RX-OK RX-ERR RX-DRP RX-OVR    TX-OK TX-ERR TX-DRP TX-OVR Flg
#eth0       1500   0 68632721      0      0      0 68034583      0      0      0 BMRU
#lo        16436   0   111578      0      0      0   111578      0      0      0 LRU

#   Iface
#       The name this interface.
#   MTU
#       Maximum size for  transmission  on  this  interface.  This
#       should  be  the size of data transferred on this interface
#       without interface specific headers.
#   RX-OK
#       error free received packets for this interface.
#   RX-ERR
#       buggy received packets.
#   RX-DRP
#       dropped received packets (due to memory lack ?).
#   RX-OVR
#       packets that we were unable to receive that fast way.
#   TX-OK
#       error free transmitted packets for this interface.
#   TX-ERR
#       buggy transmitted packets.
#   TX-DRP
#       dropped transmitteded packets.
#   TX-OVR
#       packets that we were unable to transmit.


chomp($host);
# skip the header row
@nicResults=@nicResults[1..$#nicResults];
# loop through results
foreach my $line (@nicResults){
	#remove EOF
	chomp $line;
	# split the row by column
	my @nicData=split /\s+/, $line;
	# skip row if column 3 doesn't match host or loopback
	#if ($nicData[3]=~m/($host)/ or $nicData[3]=~m/loopback/){
	if ($nicData[0]!~m/lo/ and $nicData[0]!~m/Iface/){
		# print metrics
		Wily::PrintMetric::printMetric (type		=> 'LongCounter',
										resource	=> 'NIC',
										subresource	=> $nicData[0],
										name		=> 'RX-OK',
										value		=> $nicData[3],
										);
		Wily::PrintMetric::printMetric (type		=> 'LongCounter',
										resource	=> 'NIC',
										subresource	=> $nicData[0],
										name		=> 'RX-ERR',
										value		=> $nicData[4],
										);
		Wily::PrintMetric::printMetric (type		=> 'LongCounter',
										resource	=> 'NIC',
										subresource	=> $nicData[0],
										name		=> 'RX-DRP',
										value		=> $nicData[5],
										);
		Wily::PrintMetric::printMetric (type		=> 'LongCounter',
										resource	=> 'NIC',
										subresource	=> $nicData[0],
										name		=> 'RX-OVR',
										value		=> $nicData[6],
										);
		Wily::PrintMetric::printMetric (type		=> 'LongCounter',
										resource	=> 'NIC',
										subresource	=> $nicData[0],
										name		=> 'TX-OK',
										value		=> $nicData[7],
										);
		Wily::PrintMetric::printMetric (type		=> 'LongCounter',
										resource	=> 'NIC',
										subresource	=> $nicData[0],
										name		=> 'TX-ERR',
										value		=> $nicData[8],
										);
		Wily::PrintMetric::printMetric (type		=> 'LongCounter',
										resource	=> 'NIC',
										subresource	=> $nicData[0],
										name		=> 'TX-DRP',
										value		=> $nicData[9],
										);
		Wily::PrintMetric::printMetric (type		=> 'LongCounter',
										resource	=> 'NIC',
										subresource	=> $nicData[0],
										name		=> 'TX-OVR',
										value		=> $nicData[10],
										);
	}
}
